@@ -122,6 +122,7 @@ mod handlers {
     mod destructure_tuple_binding;
     mod expand_glob_import;
     mod extract_function;
+    mod extract_module;
     mod extract_struct_from_enum_variant;
     mod extract_type_alias;
     mod extract_variable;
@@ -160,6 +161,7 @@ mod handlers {
     mod promote_local_to_const;
     mod pull_assignment_up;
     mod qualify_path;
+    mod qualify_method_call;
     mod raw_string;
     mod remove_dbg;
     mod remove_mut;
@@ -241,6 +243,7 @@ mod handlers {
             pull_assignment_up::pull_assignment_up,
             promote_local_to_const::promote_local_to_const,
             qualify_path::qualify_path,
+            qualify_method_call::qualify_method_call,
             raw_string::add_hash,
             raw_string::make_usual_string,
             raw_string::remove_hash,
@@ -275,6 +278,7 @@ mod handlers {
             //
             extract_variable::extract_variable,
             extract_function::extract_function,
+            extract_module::extract_module,
             //
             generate_getter::generate_getter,
             generate_getter::generate_getter_mut,
